/*
 disasm68k.h
*/

/* ---- values */

enum
{ /* op sizes */
	_oz_implied,
	_oz_b,	/* byte		;8-bit */
	_oz_w,	/* word		;16-bit */
	_oz_l,	/* long		;32-bit */
	_oz_s,	/* single	;32-bit FP */
	_oz_d,	/* double	;64-bit FP */
	_oz_x,	/* extended	;80-bit FP, stored w. padding after exp */
	_oz_p,	/* packed	;96-bit FP, BCD */
	_oz_c	/* constant	;6-bit index to 80-bit FP - FMOVECR only */
};

enum
{ /* instruction op-indices */
	_op_ftrap,
	_op_atrap,
	_op_itrap, /* non-A/F bad opcode */
	_op_trap,
	_op_rtm,
	_op_exg,
	_op_ext,
	_op_extb,
	_op_swap, /* 8 */
	_op_moveq,
	_op_pack,
	_op_unpk,
	_op_addx,
	_op_subx,
	_op_abcd,
	_op_sbcd,
	_op_asr, /* 16 - effective addr types */
	_op_asl,
	_op_lsr,
	_op_lsl,
	_op_roxr,
	_op_roxl,
	_op_ror,
	_op_rol,
	_op_ori, /* 24 */
	_op_andi,
	_op_subi,
	_op_addi,
	_op_eori,
	_op_cmpi,
	_op_cas,
	_op_cas2, /* in opinfo for MOVE SR/CCR */
	_op_nbcd, /* 32 */
	_op_eor,
	_op_mulu,
	_op_muls,
	_op_divu,
	_op_divs,
	_op_divul,
	_op_divsl,
	_op_btst, /* 40 */
	_op_bchg,
	_op_bclr,
	_op_bset,
	_op_tas,
	_op_move, /* largest instruction possible: 22 bytes */
	_op_movem,
	_op_move16, /* '040 only */
	_op_fmovem, /* 48 */
	_op_fnop,
	_op_fbcc,
	_op_fscc,
	_op_fsave,
	_op_frestore,
	_op_cinva,
	_op_cpusha,
	_op_pmove, /* 56 */
	_op_pflush, /* '851/'030/040 */
	_op_pbcc, /* '851 only */
	_op_pscc, /* '851 only */
	_op_psave, /* '851 only */
	_op_prestore, /* '851 only */
	_op_jsr,
	_op_jmp,
	_op_bra, /* 64 */
	_op_bsr,
	_op_bcc,
	_op_scc,
	_op_movea,
	_op_moves,
	_op_cmpa,
	_op_cmp,
	_op_cmp2, /* 72 - +8->chk2 */
	_op_tst,
	_op_addq,
	_op_subq,
	_op_adda,
	_op_suba,
	_op_add,
	_op_sub,
	_op_chk2, /* 80 */
	_op_cmpm,
	_op_clr,
	_op_not,
	_op_negx,
	_op_neg,
	_op_or,
	_op_and,
	_op_chk, /* 88 */
	_op_callm,
	_op_bkpt,
	_op_dbcc,
	_op_tcc,
	_op_trapcc, /* assemblers want Tcc and TPcc.oz */
	_op_movec,
	_op_movep,
	_op_bftst, /* 96 */
	_op_bfextu,
	_op_bfchg,
	_op_bfexts,
	_op_bfclr,
	_op_bfffo,
	_op_bfset,
	_op_bfins,
	_op_link,/* 104 */
	_op_unlk,
	_op_lea,
	_op_pea,
	_op_reset,
	_op_nop,
	_op_stop,
	_op_rte,
	_op_rtd, /* 112 */
	_op_rts,
	_op_trapv,
	_op_rtr,
	_op_ptestw,
	_op_ptestr,
	_op_ploadw, /* '030/'851 only */
	_op_ploadr, /* '030/'851 only */
	_op_plpaw, /* 120 - '060 */
	_op_plpar, /* '060 */
	_op_pflushn,
	_op_pflushan,
	_op_pflusha,
	_op_pflushr,
	_op_pflushs,
	_op_pmovefd,
	_op_pvalid, /* 128 - '851 only */
	_op_pdbcc, /* '851 only */
	_op_ptrapcc, /* '851 only */
	_op_cinvl,
	_op_cpushl,
	_op_cinvp,
	_op_cpushp,
	_op_bgnd,
	_op_tblu, /* 136 */
	_op_tblun,
	_op_tbls,
	_op_tblsn,
	_op_lpstop,
	_op_fdbcc,
	_op_ftrapcc,
	_op_fmovecr,
	_op_fmove, /* 144.. */
	_op_fsincos = 177
};

/* ---- prototypes */

/*
 disasm68K
  Disassembles one instruction, with optional name, operands and comment.

  Result:
   Size in low 8 bits; oz, ea & op-index in upper bytes.
*/
int disasm68K(const uint8 *code,
	int offs,
	int size,
	uint8 *inst, /* can be NULL */
	uint8 *oper, /* can be NULL */
	uint8 *comm); /* can be NULL */

/*
 parseMacsBugSymbol
  Verify and optionally copy a MacsBug module name, if one is present.

  Result:
   Name data length in low byte, prename length & type in next 8 bits up;
    total padding size (if any) in top 16 bits;
    all zero if no working symbol was found.
*/
int parseMacsBugModuleName(const uint8 *symbol,
	int size, /* maximum code size in reading name */
	uint16 optype, /* pass disasm68k result>>8 or _op_rts */
	char *name, /* null-terminated; can be NULL */
	int maxnamelen, /* name buffer length, not including null termination */
	char *prename); /* first assembly dc.b operand; can be NULL */

/*
 unpackMacsBugCppName
  'unmangles' a C++ symbol name.

  Result:
   1 if a valid symbol, successfully unpacked; zero if not.
*/
int unpackMacsBugCppName(char *fullname,
	int maxlen,
	const char *s);
